{ open Parser }
let integer=['0'-'9']
let dec = ((integer+ '.' integer*) | ('.' integer+))
rule token = parse
[' ' '\t' '\r' '\n'] 	{ token lexbuf } 				(* Whitespace *)
| "/*"     				{ comment lexbuf }           	(* Comments *)

| '('      				{ LPAREN }						(* Operators *)
| ')'	      			{ RPAREN }
| '{'     				{ LBRACE }
| '}'      				{ RBRACE }
| '[' 	   				{ LBRACKET }
| ']'	   				{ RBRACKET }
| ';'      				{ SEMI }
| ','      				{ COMMA }
| '+'      				{ PLUS }
| '-'      				{ MINUS }
| '*'      				{ TIMES }
| '/'      				{ DIVIDE }
| '='      				{ ASSIGN }
| ':' 					{ COLON }
| '~'	   				{ CONCAT }
| '^'	   				{ EXP }
| '!'	   				{ NOT }
| "=="     				{ EQ }
| "!="     				{ NEQ }
| '<'      				{ LT }
| '>'      				{ GT }
| "<="     				{ LEQ }
| ">="     				{ GEQ }
| "&&"	   				{ AND }
| "||"     				{ OR }

| "if"     				{ IF }							(* Statements *)
| "else"   				{ ELSE }
| "return" 				{ RETURN }
| "loop"   				{ LOOP }
| "start"  				{ START }
| "do"					{ DO }
| "conditions"			{ CONDITIONS }
| "check"				{ CHECK }
| "change"				{ CHANGE }

| "int"    				{ INT }							(* Data types *)
| "bool"   				{ BOOL }
| "double" 				{ DOUBLE }
| "string" 				{ STRING }
| "list" 				{ LI }
| "function" 			{ FUNCTION }
| "none"	 			{ NONE }
| "returns"	 			{ RETURNS }

(* literals *)
| integer+ as lxm     	     { INT_LITERAL(int_of_string lxm) }
| dec as lxm 				 { DOUBLE_LITERAL(float_of_string lxm) }
| '"' ([^'"']* as lit) '"'   { STRING_LITERAL(lit) }
| ("true" | "false") as lit	 { BOOL_LITERAL(bool_of_string lit) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
"*/" { token lexbuf }
| _    { comment lexbuf }
