open Unix
open String
open Filename
type action = Ast | Sast | JavaScript

let usage (name:string) =
  "usage:\n" ^ name ^ "\n" ^
    "        -a source.vc                 (Print AST of an vc source)\n"^
    "        -s source.vc                 (Run Semantic Analysis over source)\n"^
    "        -j source.vc [target.js]   (Generate JavaScript code for vc)\n"
  
    

let backend_path = "../backend/src/"
let target_path = backend_path ^ "com/vc/"

let read_process command =
  let buffer_size = 2048 in
  let buffer = Buffer.create buffer_size in
  let string = Bytes.create buffer_size in
  let in_channel = Unix.open_process_in command in
  let chars_read = ref 1 in
  while !chars_read <> 0 do
    chars_read := input in_channel string 0 buffer_size;
    Buffer.add_substring buffer string 0 !chars_read
  done;
  ignore (Unix.close_process_in in_channel);
  Buffer.contents buffer

(* Create a pipe for the subprocess output. *)
let readme, writeme = Unix.pipe ()

let _ =
  let args = Array.length Sys.argv in
    let action = if args > 1 then
      List.assoc Sys.argv.(1) [ ("-a", Ast); ("-s", Sast); ("-j", JavaScript);]
    else JavaScript in
      let len = String.length Sys.argv.(2)-3 in
      let jname = Sys.argv.(2) in
      let fname = String.sub jname 0 len in
      let input = open_in jname in
      let lexbuf = Lexing.from_channel input in
      let program = Parser.program Scanner.token lexbuf in
      match action with
              Ast -> let listing = Ast.string_of_program program in
                     print_string listing
            | Sast -> let env = Semantic_check.run_program program in
                           ignore env; print_string "Passed Semantic Analysis.\n"
            | JavaScript -> let sast = Semantic_check.run_program program in
                      let javaScript_source = Code_gen.prog_gen sast in
                      if args == 3 then
                        let output_file = open_out (fname ^ ".js") in
                        output_string output_file javaScript_source; close_out output_file;
                        let html_source = "<!DOCTYPE html>\n
                        <html lang=\"en-US\">\n
                        <meta charset=\"UTF-8\">\n
                        <script src=\"https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js\"></script>\n
                        <script src=\"./library.js\" type=\"text/javascript\"></script>\n
                        <script src=\"./"^basename (fname^".js")^"\" type=\"text/javascript\"></script>\n
                        <body>\n
                        <title>Test</title>\n
                        <ol id=\"content\">\n
                        </ol>\n
                        <button type=\"button\" id=\"submit\">Submit</button>\n
                        </body>\n
                        </html>" in 
                        let html_output = open_out (fname ^ ".html") in
                        output_string html_output html_source; close_out html_output;               
